%pong
% Really simple single player pong game!
%
% LAST UPDATED by Andy French April 2016

function pong
global d

%Set initial parameters

%Paddle dimensions in normalized units of the screen
d.paddle_width = 1/5;
d.paddle_height = 1/40;

%Paddle speed
d.paddle_speed = 1;

%Quit status
d.quit = 0;

%Pause status
d.pause = 0;

%Pong reset status
d.reset = 1;

%Paddle status
d.move_paddle = 0;   % -1 means move left, 0 means don't move, +1 means move right.

%Initialize score and times between losses
d.score = 0;
d.game_times = 0;

%

%Create a figure and axes which represents our pong canvas!
d.fig = figure('color',[0,0,0],'WindowButtonDownFcn',@clickmouse,...
    'Toolbar','none','menubar','none','numbertitle','off',...
    'KeyPressFcn',@keyDown, 'KeyReleaseFcn', @keyUp,...
    'CloseRequestFcn', @closeGUI);
d.ax = axes('units','normalized','color',[0,0,0],'outerposition',[0,0,1,1],...
    'position',[0,0,1,1],'xlim',[0,1],'ylim',[0,1],'nextplot','add');
axis manual

%Create paddle
d.paddle = patch( 'xdata', [ 0.5 - d.paddle_width/2,  0.5 + d.paddle_width/2, 0.5 + d.paddle_width/2, 0.5 - d.paddle_width/2 ],...
    'ydata',[ 0.1 - d.paddle_height/2,  0.1 - d.paddle_height/2, 0.1 + d.paddle_height/2, 0.1 + d.paddle_height/2 ], 'facecolor',[0,1,0] );

%Create message
d.text= text(0.5,0.5,'','fontsize',60,'color','r','HorizontalAlignment','center');

%Create pong ball
d.ball = plot( 0.5, 0.5, 'w*', 'markersize', 10 );

%Pong ball speed in screen units per second
d.ball_speed = 1;

%Time interval /s between loops
d.dt = 0.01;

%

%Run the game!
while d.quit == 0
    
    %Check if game is in pause state
    if d.pause == 0
        
        %Check if game needs to be reset
        if d.reset == 1
            
            %Reset paddle position
            set( d.paddle, 'xdata', [ 0.5 - d.paddle_width/2,  0.5 + d.paddle_width/2,...
                0.5 + d.paddle_width/2, 0.5 - d.paddle_width/2 ],...
                'ydata',[ 0.1 - d.paddle_height/2,  0.1 - d.paddle_height/2,...
                0.1 + d.paddle_height/2, 0.1 + d.paddle_height/2 ] );
            
            %Reset position of  pong ball
            set( d.ball,'xdata',0.5, 'ydata',0.5 );
            
            %Pong ball initial velocity. Initial angle is randomized
            d.vx = d.ball_speed*cos( 2*pi*rand(1,1) );
            d.vy = d.ball_speed*sin( 2*pi*rand(1,1) );
            
            %Prevent further reset in the next loop
            d.reset = 0;
        end
        
        %Get paddle x data
        paddle_x = get( d.paddle, 'xdata' );
        paddle_y = get( d.paddle, 'ydata' );
        
        %Move paddle if arrow keys have been pressed
        if ( d.move_paddle == -1 ) && ( paddle_x(1) > 0 )
            %Move left
            set( d.paddle, 'xdata', paddle_x - d.paddle_speed*d.dt );
        elseif ( d.move_paddle == 1 ) && ( paddle_x(2) < 1 )
            %Move right
            set( d.paddle, 'xdata', paddle_x + d.paddle_speed*d.dt );
        end
        
        %Get current pong ball position
        x = get( d.ball,'xdata') + d.vx * d.dt;
        y = get( d.ball,'ydata') + d.vy * d.dt;
        
        %Check for collisions with walls
        if (x > 1) || (x < 0)
            d.vx = -d.vx;
        end
        if (y > 1)
            d.vy = -d.vy;
        end
        
        %Check for collisions with paddle. For simplicity assume only y
        %component of velocity is modified following a collision.
        if ( x > paddle_x(1) ) && ( x < paddle_x(2) ) && ( y > paddle_y(1) ) && ( y < paddle_y(3) )
            
            %Bounce (i.e. vertical component of velocity)
            d.vy = -d.vy;
            
            %Swerve (i.e. horizontal component of velocity)
            if d.move_paddle == 1
                d.vx = d.vx - d.paddle_speed;
            elseif d.move_paddle == -1
                d.vx = d.vx + d.paddle_speed;
            end
        end
        
        %Update trajectory of pong ball
        set( d.ball, 'xdata', x, 'ydata', y );
        
        %Check if ball passes paddle
        if (y < 0)
            
            %Reset game
            d.reset = 1;
            
            %Update score and add a new element to the game_times vector
            d.score = d.score + 1;
            d.game_times = [d.game_times,0];
            
            %Display losing message to screen
            set( d.text, 'string', {'YOU LOSE',[num2str(d.score),' balls missed']} );
            pause(1);
            set( d.text, 'string', '');
        end
        
        %Update game time
        d.game_times(end) = d.game_times(end) + d.dt;
        
        %Update figure title with game times
        set( d.fig, 'name', ['Current game time = ',...
            num2str( round( d.game_times(end) ) ),'s. Misses = ',num2str(d.score)] );
        
        %Force screen redraw
        drawnow;
        pause(d.dt);
    end
end

%Close figure
closereq;

%%

%% SUBROUTINES %%

%Function which runs when mouse is clicked in the window. When you click in
%the pong figure window, the pong ball is replaced to the centre of the
%screen, with a random direction.
function clickmouse(fig,evnt)
global d
d.reset = 1;

%%

%Function which runs when a key is pressed. Right and left arrows move the
%paddle which will bat away the pong ball.
function keyDown( fig,evnt )
global d
if strcmp(get(fig,'currentkey'),'leftarrow')==1
    d.move_paddle = -1;
elseif strcmp(get(fig,'currentkey'),'rightarrow')==1
    d.move_paddle = 1;
elseif strcmp(get(fig,'currentkey'),'q')==1
    d.quit = 1;
elseif strcmp(get(fig,'currentkey'),'r')==1
    d.reset = 1;
    d.move_paddle = 0;
elseif strcmp(get(fig,'currentkey'),'p')==1
    if d.pause == 1
        d.pause = 0;
    else
        d.pause = 1;
    end
else
    d.move_paddle = 0;
end

%%

%Function which runs when a key is unpressed. The paddle will now stop
%moving.
function keyUp( fig,evnt )
global d
d.move_paddle = 0;

%%

%Function which runs when figure is closed
function closeGUI( fig,evnt )
global d
d.quit = 1;

%End of code